﻿#include "DIALOG.h"

/*********************************************************************
*
*       Externals
*
**********************************************************************
*/

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define THRESHOLD  20

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

static WM_HWIN _GetNearestWidget(WM_MESSAGE * pMsg, GUI_PID_STATE * pState) {
  WM_HWIN  hItem;
  GUI_RECT Rect;

  //
  // Get the first child
  //
  hItem = WM_GetFirstChild(pMsg->hWin);
  while (hItem) {
    //
    // Get its screen coordinates
    //
    WM_GetWindowRectEx(hItem, &Rect);
    //
    // Add a threshold to the rectangle
    //
    Rect.x0 -= THRESHOLD;
    Rect.y0 -= THRESHOLD;
    Rect.x1 += THRESHOLD;
    Rect.y1 += THRESHOLD;
    //
    // And check if touch is within range
    //
    if ((pState->x >= Rect.x0) && (pState->x <= Rect.x1)) {
      if ((pState->y >= Rect.y0) && (pState->y <= Rect.y1)) {
        //
        // Return the widget
        //
        return hItem;
      }
    }
    //
    // Get the next sibling of the current widget
    //
    hItem = WM_GetNextSibling(hItem);
  }
  //
  // None is found
  //
  return 0;
}

/*********************************************************************
*
*       _cbBk
*/
static void _cbBk(WM_MESSAGE * pMsg) {
  WM_MESSAGE      Message;
  WM_HWIN         hItem;
  GUI_PID_STATE * pState;

  switch (pMsg->MsgId) {
  case WM_PAINT:
    GUI_SetBkColor(GUI_BLACK);
    GUI_Clear();
    break;
  case WM_TOUCH:
    //
    // Check if touch is valid
    //
    pState = (GUI_PID_STATE *)pMsg->Data.p;
    if (pState) {
      //
      // Check for the nearest one, or for those within range
      //
      hItem = _GetNearestWidget(pMsg, pState);
      if (hItem) {
        //
        // We got one, set focus to this widget, this is not necessary for a button but other require it like the EDIT widget
        //
        WM_SetFocus(hItem);
        //
        // Pass the touch message forward to the widget
        //
        Message.Data.p = pMsg->Data.p;
        Message.MsgId  = pMsg->MsgId;
        WM_SendMessage(hItem, &Message);
      } else {
        WM_DefaultProc(pMsg);
      }
    }
    break;
  default:
    WM_DefaultProc(pMsg);
    break;
  }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       MainTask
*/
void MainTask(void) {
  WM_HWIN hItem;

  GUI_Init();
  //
  // Set callback forthe desktop window
  //
  WM_SetCallback(WM_HBKWIN, _cbBk);
  //
  // Create two widgets as childs of the desktop window
  //
  hItem = BUTTON_CreateEx(80, 80, 80, 20, WM_HBKWIN, WM_CF_SHOW, 0, GUI_ID_BUTTON0);
  BUTTON_SetText(hItem, "Text");
  hItem = EDIT_CreateEx(80, 160, 120, 30, WM_HBKWIN, WM_CF_SHOW, 0, GUI_ID_EDIT0, 10);
  EDIT_SetFont(hItem, &GUI_Font20_1);
  EDIT_SetText(hItem, "Edit");
  while (1) {
    GUI_Delay(100);
  }
}

/*************************** End of file ****************************/
